/*	$Id: 2wire.c,v 1.4 2000/04/11 17:09:41 bodo Exp $	*/
/*
    ChipDrive Library
    Copyright (C) 1997-2000  Bodo Bellut <bodo@bellutnet.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "common.h"

#ifndef lint
static char vcid[] UNUSED = "$Id: 2wire.c,v 1.4 2000/04/11 17:09:41 bodo Exp $";
#endif /* lint */

#include <string.h>
#include <stdlib.h>


int two_wire_read_errorcounter(int fd, BYTE *ec)
{
  BYTE *retval;
  BYTE status;
  BYTE tmp;
  
  retval = malloc(1*sizeof(BYTE));

  /* Status */
  write_cd(fd, 2, 0x03, 0x07);
  read_cd(fd, retval);
  status = *retval;
  read_cd(fd, retval);

  if (!((status >> 6) & 0x01)) {
    free(retval);
    return -ECD_NOCARD;
  }

  /* LED green */
  write_cd(fd, 5, 0x6f, 0x02, 0x6a, 0x0f, 0x5e);
  read_cd(fd, retval);

  /* activate */
  write_cd(fd, 3, 0x60, 0x0f, 0x9c);
  read_cd(fd, retval);
  
  write_cd(fd, 9, 0x70, 0x64, 0x42, 0x31, 0x00, 0x00, 0x65, 0x0f, 0x80);
  read_cd(fd, retval);
  write_cd(fd, 2, 0x13, 0x27);
  read_cd(fd, retval);
  tmp = *retval;
  read_cd(fd, retval);
  read_cd(fd, retval);
  read_cd(fd, retval);
  read_cd(fd, retval);

  /* deactivate */
  write_cd(fd, 3, 0x61, 0x0f, 0x98);
  read_cd(fd, retval);

  /* LED off */
  write_cd(fd, 3, 0x6b, 0x0f, 0xb0);
  read_cd(fd, retval);

  free(retval);
  
  *ec = count_bits(tmp);

  return 0;
}

int two_wire_read(int fd, /* int size, */ BYTE start, int len, BYTE *buf)
/*
  Es mu dafr gesorgt werden, da buf gro genug ist, um alle Daten zu halten.
  len = 0 bedeutet ganze Karte, start mu dann 0 sein.
  */
{
  BYTE *retval;
  BYTE status;
  int i,j;
  int pos = 0;

  if (len == 0) {
    if (start != 0)
      return -ECD_INVAL;
    len = /* size * 1024 / 8; */ 256;
  }
  
  retval = malloc(1*sizeof(BYTE));

  /* Status */
  write_cd(fd, 2, 0x03, 0x07);
  read_cd(fd, retval);
  status = *retval;
  read_cd(fd, retval);

  if (!((status >> 6) & 0x01)) {
    free(retval);
    return -ECD_NOCARD;
  }

  /* LED green */
  write_cd(fd, 5, 0x6f, 0x02, 0x6a, 0x0f, 0x5e);
  read_cd(fd, retval);

  /* activate */
  write_cd(fd, 3, 0x60, 0x0f, 0x9c);
  read_cd(fd, retval);
  
  /* INIT */
  qs = 0;
  write_qs(fd, 8, 0x70, 0x64, 0x42, 0x30,
	   start,
	   0x00, 0x65, 0x0f);
  write_qs(fd, 1, qs);
  read_cd(fd, retval);
  
  for (j=0; j < (len / 15); j++) {
    qs = 0;
    write_qs(fd, 1, 0x1e);  /* read 15 bytes */
    write_qs(fd, 1, qs);
    for (i=0; i < 15; i++) {
      read_qs(fd, retval);
      buf[pos++] = *retval;
    }
    read_cd(fd, retval);
  }

  if ((len % 15) != 0) {
    qs = 0;
    write_qs(fd, 1, ((len % 15) - 1) | 0x10);
    write_qs(fd, 1, qs);
    for (j=0; j < (len % 15); j++) {
      read_qs(fd, retval);
      buf[pos++] = *retval;
    }
    read_qs(fd, retval);
  }

  /* deactivate */
  write_cd(fd, 3, 0x61, 0x0f, 0x98);
  read_cd(fd, retval);

  /* LED off */
  write_cd(fd, 3, 0x6b, 0x0f, 0xb0);
  read_cd(fd, retval);

  free(retval);

  return pos;
}
