/*	$Id: 3wire.c,v 1.5 2000/04/11 17:09:41 bodo Exp $	*/
/*
    ChipDrive Library
    Copyright (C) 1997-2000  Bodo Bellut <bodo@bellutnet.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "common.h"

#ifndef lint
static char vcid[] UNUSED = "$Id: 3wire.c,v 1.5 2000/04/11 17:09:41 bodo Exp $";
#endif /* lint */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>


int _three_wire_read_errorcounter(int fd, BYTE *ec);
int _three_wire_enterpin(int fd, int pin);
int _three_wire_changepin(int fd, int newpin);
int _three_wire_read(int fd, int start, int len, BYTE *buf);
int _three_wire_write(int fd, int start, int len, const BYTE *buf);

int three_wire_read_errorcounter(int fd, BYTE *ec)
{
  int retval;
  
  retval = get_status(fd);
  
  if (retval == CD_STATUS_NN || retval == CD_STATUS_NC)
    return -ECD_NOCARD;
  
  retval = set_led(fd, CD_LED_GRN);
  
  retval = activate_card(fd);
  
  retval = _three_wire_read_errorcounter(fd, ec);
  
  retval = deactivate_card(fd);
  
  retval = set_led(fd, CD_LED_OFF);
  
  return 0;
}


int _three_wire_read_errorcounter(int fd, BYTE *ec)
{
  BYTE retval;
  
  write_cd(fd, 10, 0x70, 0xa0, 0x42, 0xce, 0xfd, 0xfd, 0x80, 0x50, 0x0f, 0x17);
  read_cd(fd, &retval);
  write_cd(fd, 2, 0x10, 0x21);
  read_cd(fd, ec);
  read_cd(fd, &retval);

  return 0;
}

BYTE clear_one_bit(const BYTE ec)
{
  int i;

  for (i = 0; i < 8; i++) {
    if ((ec >> i) & 1) {
      return (ec & (~ (1 << i)));
    }
  }
  return 0;
}

int three_wire_enterpin(int fd, int pin)
{
  int retval;
  
  retval = get_status(fd);
  
  if (retval == CD_STATUS_NN || retval == CD_STATUS_NC)
    return -ECD_NOCARD;
  
  retval = set_led(fd, CD_LED_GRN);
  
  retval = activate_card(fd);
  
  retval = _three_wire_enterpin(fd, pin);
  
  retval = deactivate_card(fd);
  
  retval = set_led(fd, CD_LED_OFF);

  return 0;
}

int _three_wire_enterpin(int fd, int pin)
{
  BYTE retval;
  BYTE ec;  

  (void) _three_wire_read_errorcounter(fd, &ec);
  
  if (count_bits(ec) < 1)
    return -ECD_LOCKED;
  
  ec = clear_one_bit(ec);
  
  /* 1: EC */
  qs = 0;
  write_qs(fd, 8, 0x73, 0x67, 0x6e, 0xfd, 0xf2, 0x02, 0x0f, 0x8c);
  read_qs(fd, &retval);

  qs = 0;
  write_qs(fd, 3, 0x40, ec, 0x0f);
  write_qs(fd, 1, qs);
  read_qs(fd, &retval);
  
  /* 2: HI(pin) */
  qs = 0;
  write_qs(fd, 8, 0x73, 0x67, 0x6e, 0xfe, 0xcd, 0x02, 0x0f, 0x45);
  read_qs(fd, &retval);

  qs = 0;
  write_qs(fd, 3, 0x40, HI(pin), 0x0f);
  write_qs(fd, 1, qs);
  read_qs(fd, &retval);
  
  /* 3: LO(pin) */
  qs = 0;
  write_qs(fd, 8, 0x73, 0x67, 0x6e, 0xff, 0xcd, 0x02, 0x0f, 0x55);
  read_qs(fd, &retval);

  qs = 0;
  write_qs(fd, 3, 0x40, LO(pin), 0x0f);
  write_qs(fd, 1, qs);
  read_qs(fd, &retval);
  
  /* 4 */
  qs = 0;
  write_qs(fd, 8, 0x73, 0x67, 0x6e, 0xfd, 0xf3, 0x02, 0x0f, 0x84);
  read_qs(fd, &retval);

  qs = 0;
  write_qs(fd, 4, 0x40, 0xff, 0x0f, 0xe4);
  read_qs(fd, &retval);
  
  return 0;
}

int three_wire_changepin(int fd, int oldpin, int newpin)
{
  int retval;
  
  retval = get_status(fd);
  
  if (retval == CD_STATUS_NN || retval == CD_STATUS_NC)
    return -ECD_NOCARD;
  
  retval = set_led(fd, CD_LED_RED);
  
  retval = activate_card(fd);
  
  retval = _three_wire_enterpin(fd, oldpin);

  retval = _three_wire_changepin(fd, newpin);
  
  retval = deactivate_card(fd);
  
  retval = set_led(fd, CD_LED_OFF);
  
  return 0;
}

int _three_wire_changepin(int fd, int newpin)
{
  BYTE retval;
  
  write_cd(fd, 8, 0x73, 0x67, 0x6e, 0xfe, 0xf3, 0x02, 0x0f, 0xb4);
  read_cd(fd, &retval);
  qs = 0;
  write_qs(fd, 3, 0x40, LO(newpin), 0x0f);
  write_cd(fd, qs);
  read_cd(fd, &retval);
  
  write_cd(fd, 8, 0x73, 0x67, 0x6e, 0xff, 0xf3, 0x02, 0x0f, 0xa4);
  read_cd(fd, &retval);
  qs = 0;
  write_qs(fd, 3, 0x40, HI(newpin), 0x0f);
  write_cd(fd, qs);
  read_cd(fd, &retval);

  return 0;
}

int three_wire_read(int fd, int start, int len, BYTE *buf)
{
  int retval;
  
  retval = get_status(fd);
  
  if (retval == CD_STATUS_NN || retval == CD_STATUS_NC)
    return -ECD_NOCARD;
  
  retval = set_led(fd, CD_LED_GRN);
  
  retval = activate_card(fd);
  
  retval = _three_wire_read(fd, start, len, buf);
  
  retval = deactivate_card(fd);
  
  retval = set_led(fd, CD_LED_OFF);
  
  return 0;
}

int _three_wire_read(int fd, int start, int len, BYTE *buf)
/*
  Es mu dafr gesorgt werden, da buf gro genug ist, um alle Daten zu halten.
  len = 0 bedeutet ganze Karte, start mu dann 0 sein.
  */
{
  BYTE retval;
  int i,j;
  int pos = 0;

  if (len == 0) {
    if (start != 0)
      return -ECD_INVAL;
    len = 1021;
  }
  
  /* INIT */
  qs = 0;
  write_qs(fd, 9, 0x70, 0xa0, 0x42,
	   (HI(start) << 6) | 0x0e,
	   LO(start),
	   0x00, 0x80, 0x50, 0x0f);
  write_qs(fd, 1, qs);
  read_cd(fd, &retval);
  
  for (j=0; j < (len / 15); j++) {
    qs = 0;
    write_qs(fd, 1, 0x1e);  /* read 15 bytes */
    write_qs(fd, 1, qs);
    for (i=0; i < 15; i++) {
      read_qs(fd, &retval);
      buf[pos++] = retval;
    }
    read_cd(fd, &retval);
  }

  if ((len % 15) != 0) {
    qs = 0;
    write_qs(fd, 1, ((len % 15) - 1) | 0x10);
    write_qs(fd, 1, qs);
    for (j=0; j < (len % 15); j++) {
      read_qs(fd, &retval);
      buf[pos++] = retval;
    }
    read_qs(fd, &retval);
  }

  return pos;
}

int three_wire_write(int fd, int start, int len, int pin, const BYTE *buf)
{
  int retval;
  
  retval = get_status(fd);
  
  if (retval == CD_STATUS_NN || retval == CD_STATUS_NC)
    return -ECD_NOCARD;
  
  retval = set_led(fd, CD_LED_RED);
  
  retval = activate_card(fd);
  
  retval = _three_wire_enterpin(fd, pin);

  retval = _three_wire_write(fd, start, len, buf);
  
  retval = deactivate_card(fd);
  
  retval = set_led(fd, CD_LED_OFF);
  
  return 0;
}

int _three_wire_write(int fd, int start, int len, const BYTE *buf)
/*
  len = 0 bedeutet ganze Karte, start mu dann 0 sein.
  */
{
  BYTE retval;
  int i,j;
  int pos = 0;

  if (len == 0) {
    if (start != 0)
      return -ECD_INVAL;
    len = /* size * 1024 / 8; */ 1021;
  }

  /* write 3-wire starting at start */
  write_cd(fd, 8, 0x73, 0x67, 0x6e,
           LO(start),
           (HI(start) << 6) | 0x33,
           0x02, 0x0f, 0x5f);
  read_cd(fd, &retval);

   /* write N bytes */
  for (j=0; j < (len / 15); j++) {
    qs = 0;
    write_qs(fd, 1, 0x4e);
    for (i=0; i<15; i++)
      write_qs(fd, 1, buf[pos++]);
    write_qs(fd, 1, qs);
    read_cd(fd, &retval);
  }

  if ((len % 15) != 0) {
    qs = 0;
    write_qs(fd, 1, ((len % 15) - 1) | 0x40);
    for (j=0; j < (len % 15); j++)
      write_qs(fd, 1, buf[pos++]);
    write_qs(fd, 1, 0x0f);
    write_qs(fd, 1, qs);
    read_cd(fd, &retval);
  }

  return pos;
}
