/* 	$Id: common.c,v 1.12 2000/04/11 17:09:41 bodo Exp $	 */
/*
    ChipDrive Library
    Copyright (C) 1997-2000  Bodo Bellut <bodo@bellutnet.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdlib.h>

#include "common.h"

#ifndef lint
static char vcid[] UNUSED = "$Id: common.c,v 1.12 2000/04/11 17:09:41 bodo Exp $";
#endif /* lint */

BYTE qs = 0;

static struct termios saved_tio[1];

void close_tty(int fd)
{
  tcsetattr(fd, TCSANOW, saved_tio);
  close(fd);
}

int init_tty(char *tty)
{
  int fd;
  struct termios tio[1];
  BYTE retval;

  if ((fd = open(tty, O_RDWR)) < 0)
    return fd;
  if (tcgetattr(fd, saved_tio) < 0) {
    close(fd);
    return -1;
  }
  tio->c_iflag = 0;
  tio->c_oflag = 0;
  tio->c_cflag = CS8|CSTOPB|CREAD|CLOCAL|PARENB;
  tio->c_lflag = NOFLSH;
  tio->c_cc[VMIN] = 1;
  tio->c_cc[VTIME] = 0;
  cfsetospeed(tio, B9600);
  if (tcsetattr(fd, TCSANOW, tio) < 0) {
    close(fd);
    return -1;
  }

  write_cd(fd, 2, 0x00, 0x01);
  read_cd(fd, &retval);
#if 0
  if (retval == 0x00) {
    write_cd(fd, 2, 0x00, 0x01);
    read_cd(fd, &retval);
  }
#endif
  read_cd(fd, &retval);
  read_cd(fd, &retval);

  write_cd(fd, 2, 0x02, 0x05);
  read_cd(fd, &retval);
  read_cd(fd, &retval);

  write_cd(fd, 3, 0x6b, 0x0f, 0xb0);
  read_cd(fd, &retval);

  return fd;
}

void calc_qs(const BYTE c)
{
  BYTE tmp, tmp2;

  tmp = qs^c;
  tmp2 = (tmp & 0x80) >> 7;
  tmp = (tmp << 1) | tmp2;
  qs = tmp^0x01;
}

int write_qs(int fd, const unsigned int len, ...)
{
  va_list args;
  BYTE p;
  unsigned int i;
  int retval = 0;
  int written = 0;

  va_start(args, len);
  for (i = 0; ((i < len) && (retval >= 0)); i++) {
    p = va_arg(args, BYTE);
    calc_qs(p);
    if ((retval = write(fd, &p, 1)) >= 0)
      written++;
  }
  va_end(args);

  if (retval < 0)
    return retval;
  else
    return written;
}

int read_qs(int fd, char *buf)
{
  int tmp;

  tmp = read(fd, buf, 1);
  calc_qs(buf[0]);
  return tmp;
}

int write_cd(int fd, const unsigned int len, ...)
{
  va_list args;
  BYTE p;
  unsigned int i;
  int retval = 0;
  int written = 0;

  va_start(args, len);
  for (i = 0; ((i < len) && (retval >= 0)); i++) {
    p = va_arg(args, BYTE);
    if ((retval = write(fd, &p, 1)) >= 0)
      written++;
  }
  va_end(args);

  if (retval < 0)
    return retval;
  else
    return written;
}

/*************** Terminal commands ***************************/

int get_status(int fd)
{
  BYTE retval;
  BYTE status;
  
  write_cd(fd, 2, 0x03, 0x07);
  read_cd(fd, &status);
  read_cd(fd, &retval);
  
  if (((status >> 4) & 0x03) != 0)
    return -status;

  status = (status >> 6) & 0x03;
  
  if (status == 0x00)
    return CD_STATUS_NN;
  else if (status == 0x01)
    return CD_STATUS_CN;
  else if (status == 0x02)
    return CD_STATUS_NC;
  else
    return CD_STATUS_CC;
}

int activate_card(int fd)
{
  BYTE retval;
  
  write_cd(fd, 3, 0x60, 0x0f, 0x9c);
  read_cd(fd, &retval);
  
  return (retval == 0x01);
}

int deactivate_card(int fd)
{
  BYTE retval;
  
  write_cd(fd, 3, 0x61, 0x0f, 0x98);
  read_cd(fd, &retval);
  
  return (retval == 0x01);
}

int set_led(int fd, cd_ledstate state)
{
  BYTE retval;
  
  switch (state) {
    case CD_LED_OFF: write_cd(fd, 3, 0x6b, 0x0f, 0xb0);             break;
    case CD_LED_RED: write_cd(fd, 5, 0x6f, 0x01, 0x6a, 0x0f, 0x46); break;
    case CD_LED_GRN: write_cd(fd, 5, 0x6f, 0x02, 0x6a, 0x0f, 0x5e); break;
    case CD_LED_YEL: write_cd(fd, 5, 0x6f, 0x03, 0x6a, 0x0f, 0x56); break;
    case CD_LED_ON : write_cd(fd, 3, 0x6a, 0x0f, 0xb4);             break;
    default        : return -ECD_INVAL;                             break;
  }
  read_cd(fd, &retval);

  return (retval == 0x01);
}
