/*	$Id: common.h,v 1.22 2000/04/11 17:09:41 bodo Exp $	*/
/*
    ChipDrive Library
    Copyright (C) 1997-2000  Bodo Bellut <bodo@bellutnet.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef COMMON_H
#define COMMON_H

#include <unistd.h>

#define ECD_NOCARD  1  /* no card in reader */
#define ECD_INVAL   2  /* invalid parameters */
#define ECD_NOSYS   3  /* function/protocoll not implemented */
#define ECD_LOCKED  4  /* card/range locked */

#define CD_PROT_UNKNOWN  0
#define CD_PROT_I2C      1
#define CD_PROT_2WIRE    2
#define CD_PROT_3WIRE    3

#define CD_STATUS_NN  1  /* No card, No change */
#define CD_STATUS_NC  2  /* No card, Change */
#define CD_STATUS_CN  3  /* Card, No change */
#define CD_STATUS_CC  4  /* Card, Change */

#define LO(a) (a & 0x00ff)
#define HI(a) ((a & 0xff00) >> 8)

#ifdef __GNUC__
#  define UNUSED __attribute__((unused))
#else
#  define UNUSED
#endif

typedef unsigned char BYTE;

typedef struct kvk_struct {
  char kkName[29];
  char kkNum[8];
  char VKNR[6];
  char vNum[13];
  char vStat[5];
  char statOW[4];
  char titel[16];
  char vorname[29];
  char namZus[16];
  char famName[29];
  char gebDat[9];
  char strNum[29];
  char wohnSLC[4];
  char PLZ[8];
  char ort[23];
  char gueltig[5];
  char checksum;
} t_kvk;

typedef struct tele_struct {
  int startup;
  int manfct;
  int code;
  int monat;
  int jahr4;
  char serNum[6];
  int wert;
} t_tele;

typedef enum {
  CD_LED_OFF = 0,  /* 0 */
  CD_LED_RED,      /* 1 */
  CD_LED_GRN,      /* 2 */
  CD_LED_YEL,      /* 3 */
  CD_LED_ON        /* 4 */  /* for monochrome LED's */
} cd_ledstate;


/* common.c */
extern BYTE qs;
/* tele.c */
extern const char *tele_manfct[];
extern const char *tele_code[];

/* Prototypen fr alle */

/* common.c */
int init_tty(char *tty);
void close_tty(int fd);
int get_status(int fd);
int activate_card(int fd);
int deactivate_card(int fd);
int set_led(int fd, cd_ledstate state);
/* atr.c */
int atr(int fd, BYTE *buf);
/* i2c.c */
int i2c_read(int fd, int size, int start, int len, BYTE *buf);
int i2c_write(int fd, int size, int start, int len, const BYTE *buf);
int i2c_verify(int fd, int size, int start, int len, const BYTE *buf);
/* kvk.c */
int kvk_read(int tty, t_kvk *kvk);
/* tele.c */
int tele_read(int fd, t_tele *tele);
int count_bits(const BYTE buf);
/* 3wire.c */
int three_wire_read_errorcounter(int fd, BYTE *ec);
int three_wire_read(int fd, int start, int len, BYTE *buf);
int three_wire_write(int fd, int start, int len, int pin, const BYTE *buf);
int three_wire_verify(int fd, int start, int len, const BYTE *buf);
int three_wire_enterpin(int fd, int pin);
int three_wire_changepin(int fd, int oldpin, int newpin);
/* 2wire.c */
int two_wire_read_errorcounter(int fd, BYTE *ec);
int two_wire_read(int fd, /* int size, */ BYTE start, int len, BYTE *buf);
int two_wire_write(int fd, /* int size, */ BYTE start, int len, const BYTE *buf);
int two_wire_verify(int fd, /* int size, */ BYTE start, int len, const BYTE *buf);
int two_wire_enterpin(int fd, int pin);

/* 'interne' Prototypen */

/* common.c */
int write_qs(int fd, const unsigned int len, ...);
int read_qs(int fd, char *buf);
int write_cd(int fd, const unsigned int len, ...);
#define read_cd(fd, buf)  read(fd, buf, 1)

#endif /* COMMON_H */
