/*	$Id: i2c.c,v 1.11 2000/04/11 17:09:41 bodo Exp $	*/
/*
    ChipDrive Library
    Copyright (C) 1997-2000  Bodo Bellut <bodo@bellutnet.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>

#include "common.h"

#ifndef lint
static char vcid[] UNUSED = "$Id: i2c.c,v 1.11 2000/04/11 17:09:41 bodo Exp $";
#endif /* lint */


int i2c_verify(int fd, int size, int start, int len, const BYTE *buf)
/*
  len = 0 bedeutet ganze Karte, start mu dann 0 sein.
  
  Rckgabe: 0:  identisch
            1:  unterschiedlich
            -n: Fehler n
  */
{
  BYTE *retval;
  BYTE status;
  int quit = 0;
  int i,j;
  int pos = 0;

  if (len == 0) {
    if (start != 0)
      return -ECD_INVAL;
    len = size * 1024 / 8;
  }

  retval = malloc(1*sizeof(BYTE));

  /* Status */
  write_cd(fd, 2, 0x03, 0x07);
  read_cd(fd, retval);
  status = *retval;
  read_cd(fd, retval);

  if (!((status >> 6) & 0x01)) {
    free(retval);
    return -ECD_NOCARD;
  }

  /* LED green */
  write_cd(fd, 5, 0x6f, 0x02, 0x6a, 0x0f, 0x5e);
  read_cd(fd, retval);

  /* activate */
  write_cd(fd, 3, 0x60, 0x0f, 0x9c);
  read_cd(fd, retval);

  /* read i2c starting at start */
  qs = 0;
  if (size < 32)
    write_qs(fd, 9, 0x7c, 0x64, 0x41,
	     (HI(start) << 1) | 0xa0,
	     LO(start) , 0x64, 0x40,
	     ((HI(start) << 1) | 0xa0) | 0x01 , 0x0f);
  else
   write_qs(fd, 10, 0x7c, 0x64, 0x42, 0xa0,
	    HI(start), LO(start), 0x64, 0x40, 0xa1, 0x0f); 
  write_qs(fd, 1, qs);
  read_cd(fd, retval);

  /* read N bytes */
  for (j=0; ((j < (len / 15)) && (!quit)); j++) {
    qs = 0;
    write_qs(fd, 1, 0x1e);
    write_qs(fd, 1, qs);
    for (i=0; i < 15; i++) {
      read_qs(fd, retval);
      if (buf[pos++] != *retval)
        quit = 1;
    }
    read_cd(fd, retval);
  }

  if (((len % 15) != 0) && (!quit)) {
    qs = 0;
    write_qs(fd, 1, ((len % 15) - 1) | 0x10);
    write_qs(fd, 1, qs);
    for (j=0; j < (len % 15); j++) {
      read_qs(fd, retval);
      if (buf[pos++] != *retval)
        quit = 1;
    }
    read_qs(fd, retval);
  }

  /* ? */
  write_cd(fd, 4, 0x58, 0x65, 0x0f, 0x4e);
  read_cd(fd, retval);
  
  /* deactivate */
  write_cd(fd, 3, 0x61, 0x0f, 0x98);
  read_cd(fd, retval);

  /* LED off */
  write_cd(fd, 3, 0x6b, 0x0f, 0xb0);
  read_cd(fd, retval);

  free(retval);

  return quit;
}


int i2c_read(int fd, int size, int start, int len, BYTE *buf)
/*
  Es mu dafr gesorgt werden, da buf gro genug ist, um alle Daten zu halten.
  len = 0 bedeutet ganze Karte, start mu dann 0 sein.
  */
{
  BYTE *retval;
  BYTE status;
  int i,j;
  int pos = 0;

  if (len == 0) {
    if (start != 0)
      return -ECD_INVAL;
    len = size * 1024 / 8;
  }

  retval = malloc(1*sizeof(BYTE));

  /* Status */
  write_cd(fd, 2, 0x03, 0x07);
  read_cd(fd, retval);
  status = *retval;
  read_cd(fd, retval);

  if (!((status >> 6) & 0x01)) {
    free(retval);
    return -ECD_NOCARD;
  }

  /* LED green */
  write_cd(fd, 5, 0x6f, 0x02, 0x6a, 0x0f, 0x5e);
  read_cd(fd, retval);

  /* activate */
  write_cd(fd, 3, 0x60, 0x0f, 0x9c);
  read_cd(fd, retval);

  /* read i2c starting at start */
  qs = 0;
  if (size < 32)
    write_qs(fd, 9, 0x7c, 0x64, 0x41,
	     (HI(start) << 1) | 0xa0,
	     LO(start) , 0x64, 0x40,
	     ((HI(start) << 1) | 0xa0) | 0x01 , 0x0f);
  else
   write_qs(fd, 10, 0x7c, 0x64, 0x42, 0xa0,
	    HI(start), LO(start), 0x64, 0x40, 0xa1, 0x0f); 
  write_qs(fd, 1, qs);
  read_cd(fd, retval);

  /* read N bytes */
  for (j=0; j < (len / 15); j++) {
    qs = 0;
    write_qs(fd, 1, 0x1e);
    write_qs(fd, 1, qs);
    for (i=0; i < 15; i++) {
      read_qs(fd, retval);
      buf[pos++] = *retval;
    }
    read_cd(fd, retval);
  }

  if ((len % 15) != 0) {
    qs = 0;
    write_qs(fd, 1, ((len % 15) - 1) | 0x10);
    write_qs(fd, 1, qs);
    for (j=0; j < (len % 15); j++) {
      read_qs(fd, retval);
      buf[pos++] = *retval;
    }
    read_qs(fd, retval);
  }

  /* ? */
  write_cd(fd, 4, 0x58, 0x65, 0x0f, 0x4e);
  read_cd(fd, retval);
  
  /* deactivate */
  write_cd(fd, 3, 0x61, 0x0f, 0x98);
  read_cd(fd, retval);

  /* LED off */
  write_cd(fd, 3, 0x6b, 0x0f, 0xb0);
  read_cd(fd, retval);

  free(retval);

  return pos;
}

int i2c_write(int fd, int size, int start, int len, const BYTE *buf)
/*
  len = 0 bedeutet ganze Karte, start mu dann 0 sein.
  */
{
  BYTE *retval;
  BYTE status;
  int i,j;
  int pos = 0;

  if (len == 0) {
    if (start != 0)
      return -ECD_INVAL;
    len = size * 1024 / 8;
  }

  retval = malloc(1*sizeof(BYTE));

  /* Status */
  write_cd(fd, 2, 0x03, 0x07);
  read_cd(fd, retval);
  status = *retval;
  read_cd(fd, retval);

  if (!((status >> 6) & 0x01)) {
    free(retval);
    return -ECD_NOCARD;
  }

  /* LED red */
  write_cd(fd, 5, 0x6f, 0x01, 0x6a, 0x0f, 0x46);
  read_cd(fd, retval);

  /* activate */
  write_cd(fd, 3, 0x60, 0x0f, 0x9c);
  read_cd(fd, retval);

  /* write i2c starting at start */
  if (size < 32)
    write_cd(fd, 10, 0x7c, 0x64, 0x41, 0xa0, 0x00, 0x64, 0x40, 0xa1, 0x0f, 0x36);
  else
    write_cd(fd, 11, 0x7c, 0x64, 0x42, 0xa0, 0x00, 0x00,  0x64, 0x40, 0xa1, 0x0f, 0x0b);
  read_cd(fd, retval);

  /* weiter INIT */
  write_cd(fd, 3, 0x7e, 0x10, 0xda);
  read_cd(fd, retval);
  read_cd(fd, retval);

  /* weiter INIT */
  qs = 0;
  if (size < 32)
    write_qs(fd, 7, 0x7e, 0x66, 0x6e, LO(start), (HI(start) << 1) | 0xa0, 0x02, 0x0f);
  else
    write_qs(fd, 7, 0x7f, 0x66, 0x6e, LO(start), HI(start), 0xa0, 0x0f);
  write_qs(fd, 1, qs);
  read_cd(fd, retval);

   /* write N bytes */
  for (j=0; j < (len / 15); j++) {
    qs = 0;
    write_qs(fd, 1, 0x4e);
    for (i=0; i<15; i++)
      write_qs(fd, 1, buf[pos++]);
    write_qs(fd, 1, qs);
    read_cd(fd, retval);
  }

  if ((len % 15) != 0) {
    qs = 0;
    write_qs(fd, 1, ((len % 15) - 1) | 0x40);
    for (j=0; j < (len % 15); j++)
      write_qs(fd, 1, buf[pos++]);
    write_qs(fd, 1, 0x0f);
    write_qs(fd, 1, qs);
    read_cd(fd, retval);
  }

  /* ? */
  write_cd(fd, 4, 0x58, 0x65, 0x0f, 0x4e);
  read_cd(fd, retval);

  /* deactivate */
  write_cd(fd, 3, 0x61, 0x0f, 0x98);
  read_cd(fd, retval);

  /* LED off */
  write_cd(fd, 3, 0x6b, 0x0f, 0xb0);
  read_cd(fd, retval);

  free(retval);

  return pos;
}
